#include <cstdlib>
#include <cstring>
#include <exception>
#include <iostream>

#include "ghostmp_common_parameters.h"
#include "common_parameters.h"
#include "db_creator.h"
#include "db_creator_parameters.h"

void PrintUsage() {
  std::cerr <<
    "ghostmp_makedb - convert a FASTA file to GHOSTX format database files. version "
    + ghostmp::kVersion + ".\n"
    "(based on GHOSTX-" + CommonParameters::kVersion +
    ", serial homology search tool.)\n" << std::endl;
  std::cerr <<
    "  ghostmp_makedb [-i dbFastaFile] [-o dbName] [-l chunkSize]\n"
    "\n"
    "  Options:\n"
    "  (Required)\n"
    "    -i STR    Protein sequences in FASTA format for a database\n"
    "    -o STR    The name of database\n"
    "\n"
    "  (Optional)\n"
    "    -l INT    Chunk size of the database (bytes) [1073741824 (=1GB)]\n"
    "    -t STR    Database sequence type, p (protein) or d (dna) [p]"
    << std::endl;
}

int main(int argc, char* argv[]) {
  if (argc < 2 ||
      strcmp(argv[1], "-h") == 0 ||
      strcmp(argv[1], "--help") == 0) {
    PrintUsage();
    exit(1);
  }
  try {
    DbCreatorParameters parameters;
    parameters.SetParameters(argc, argv);
    DbCreator db_creator;
    db_creator.Run(parameters);
  } catch (std::exception &e) {
    std::cerr << e.what();
    exit(1);
  }
  return EXIT_SUCCESS;
}

