/*
 * query.cpp
 *
 *  Created on: 2009/10/09
 *      Author: shu
 */

#include "aligner.h"
#include "aligner_parameters.h"
#include "sequence_reader.h"
#include "sequence.h"
#include "encoder.h"
#include "dna_sequence.h"
#include "dna_type.h"
#include "dna_query.h"
#include "protein_type.h"
#include "statistics.h"
#include "translator.h"
#include "translated_dna_query.h"
#include "protein_query.h"
#include "protein_type.h"
#include "queries.h"
#include <cmath>
#include <cstddef>
#include <algorithm>
#include <numeric>
#include <typeinfo>
#include <string>
#include <stdint.h>
#include <vector>
#include <iostream>
#include <stdexcept>
#include <utility>
#include <functional>

using namespace std;

Queries::Queries(const AlignerParameters &parameters) :
  parameters_(parameters),
  max_sequence_length_(0),
  reader_(CreateSequenceReader(parameters.GetQueriesFileNmae().c_str(), FASTA)),
  next_sequence_(SequencePtr())
{
  Next();
}

void Queries::Next() {
  Clear();
  max_sequence_length_ = 0;
  uint32_t sum_length= 0;

  if(!next_sequence_) {
    next_sequence_ = reader_->Read();
  }

  if (!next_sequence_) {
    return;
  }

  uint32_t max_sequences_length = parameters_.GetMaxQueriesLength();

  while(GetLength(next_sequence_) <= 0) {
    next_sequence_ = reader_->Read();
    if (!next_sequence_) {
      return;
    }
  }

  if (GetLength(next_sequence_) >= max_sequences_length) {
    throw invalid_argument("the max queries length is too short.");
  }

  while (next_sequence_) {
    if (GetLength(next_sequence_) > 0) {
      if (sum_length + GetLength(next_sequence_) >= max_sequences_length) {
        break;
      }
      sum_length += GetLength(next_sequence_);
      if (max_sequence_length_ < GetLength(next_sequence_)) {
        max_sequence_length_ = GetLength(next_sequence_);
      }
      SetQuery(next_sequence_);
    }
    next_sequence_ = reader_->Read();
  }
}

uint32_t Queries::GetLength(SequencePtr sequence) {
  if (typeid(*(parameters_.GetAlignedType())) == typeid(ProteinType)) {
    if (parameters_.Translated()) {
      return TranslatedDnaQuery::GetSequenceLength(sequence->GetSequenceData().size());
    } else {
      return ProteinQuery::GetSequenceLength(sequence->GetSequenceData().size());
    }
  } else {
    return DnaQuery::GetSequenceLength(sequence->GetSequenceData().size());
  }
}

void Queries::SetQuery(SequencePtr sequence) {
  if (typeid(*(parameters_.GetAlignedType())) == typeid(ProteinType)) {
    if (parameters_.Translated()) {
      queries_.push_back(new TranslatedDnaQuery(*sequence, parameters_));
    } else {
      queries_.push_back(new ProteinQuery(*sequence, parameters_));
    }
  } else {
    queries_.push_back(new DnaQuery(*sequence, parameters_));
  }
}

void Queries::Clear() {
  for (vector<Query *>::iterator it = queries_.begin(); it != queries_.end(); ++it) {
    delete *it;
  }
  queries_.clear();
}
