#include <gtest/gtest.h>
#include <fstream>
#include "../src/suffix_array.h"

TEST(SuffixArrayTest, GetMaxAlphabet) {
  unsigned char text[] = {0, 1, 1, 2, 4};
  unsigned int text_length = 5;
  SuffixArray sa(text, text_length);
  EXPECT_EQ(4, sa.GetMaxAlphabet());
}

TEST(SuffixArrayTest, SearchOneCharacter)
{
	unsigned char text[] = "abbcd";
	unsigned int text_length = 5;
	SuffixArray sa(text, text_length);
	unsigned int start = 0;
	unsigned int end = text_length - 1;

	start = 0;
	end = text_length - 1;
  sa.Search('a', 0, &start, &end);

  EXPECT_EQ(0, start);
  EXPECT_EQ(0, end);

  start = 0;
  end = text_length - 1;
  sa.Search('b', 0, &start, &end);

  EXPECT_EQ(1, start);
  EXPECT_EQ(2, end);

  start = 0;
  end = text_length - 1;
	sa.Search('d', 0, &start, &end);

	EXPECT_EQ(4, start);
	EXPECT_EQ(4, end);
}

TEST(SuffixArrayTest, SearchAllCharacters)
{
  unsigned char text[] = {1, 4, 1, 2, 4};
  unsigned int text_length = 5;
  SuffixArray sa(text, text_length);
  unsigned int start = 0;
  unsigned int end = text_length - 1;
  unsigned int start_list[sa.GetMaxAlphabet() + 1];
  unsigned int end_list[sa.GetMaxAlphabet() + 1];
  unsigned char start_c = 1;
  unsigned char end_c = 4;
  sa.Search(0, start, end, &start_c, &end_c, start_list, end_list);
  EXPECT_EQ(0, start_list[1]);
  EXPECT_EQ(1, end_list[1]);
  EXPECT_EQ(2, start_list[2]);
  EXPECT_EQ(2, end_list[2]);
  ASSERT_FALSE(start_list[3] <= end_list[3]);
  EXPECT_EQ(3, start_list[4]);
  EXPECT_EQ(4, end_list[4]);
}

TEST(SuffixArrayTest, SaveAndLoad)
{
  unsigned char text[] = {1, 4, 1, 2, 4};
  unsigned int text_length = 5;
  SuffixArray sa_base(text, text_length);
  char filename[] = "test.sa";
  ofstream os;
  os.open(filename);
  sa_base.Save(os);
  os.close();

  ifstream is;
  is.open(filename);
  SuffixArray sa(is);
  is.close();
  unsigned int start = 0;
  unsigned int end = text_length - 1;
  unsigned int start_list[sa.GetMaxAlphabet() + 1];
  unsigned int end_list[sa.GetMaxAlphabet() + 1];
  unsigned char start_c = 1;
  unsigned char end_c = 4;
  sa.Search(0, start, end, &start_c, &end_c, start_list, end_list);
  EXPECT_EQ(0, start_list[1]);
  EXPECT_EQ(1, end_list[1]);
  EXPECT_EQ(2, start_list[2]);
  EXPECT_EQ(2, end_list[2]);
  ASSERT_FALSE(start_list[3] <= end_list[3]);
  EXPECT_EQ(3, start_list[4]);
  EXPECT_EQ(4, end_list[4]);
}

